@@ -417,6 +417,14 @@ swf_tag_get_refcid(swf_tag_t *tag) {
             return -1;
         }
         return swf_tag_place->character_id;
+    } else if (tag->code == 37) { // DefineEditText
+        swf_tag_edit_detail_t *swf_tag_edit;
+        swf_tag_edit = swf_tag_create_input_detail(tag, NULL);
+        if (swf_tag_edit == NULL) {
+            fprintf(stderr, "swf_tag_get_refcid: swf_tag_edit swf_tag_create_input_detail failed\n");
+            return -1;
+        }
+        return swf_tag_edit->edit_font_id_ref;
     }
     return -1; // no cid tag
 }
@@ -435,6 +443,14 @@ swf_tag_replace_refcid(swf_tag_t *tag, int cid) {
             return 1; // failure
         }
         swf_tag_place->character_id = cid;
+    } else if (tag->code == 37) { // DefineEditText
+        swf_tag_edit_detail_t *swf_tag_edit;
+        swf_tag_edit = swf_tag_create_input_detail(tag, NULL);
+        if (swf_tag_edit == NULL) {
+            fprintf(stderr, "swf_tag_get_refcid: swf_tag_edit swf_tag_create_input_detail failed\n");
+            return -1;
+        }
+        swf_tag_edit->edit_font_id_ref = cid;
     }
     if (tag->data) {
         free(tag->data);
@@ -966,7 +982,7 @@ swf_tag_replace_edit_string(swf_tag_t *tag,
 }
 
 int
-swf_tag_apply_shape_matrix_factor(swf_tag_t *tag, int shape_id,
+swf_tag_apply_shape_matrix_factor(swf_tag_t *tag, int shape_id, int bitmap_id,
                                   double scale_x, double scale_y,
                                   double rotate_rad,
                                   signed int trans_x, signed int trans_y,
@@ -989,9 +1005,10 @@ swf_tag_apply_shape_matrix_factor(swf_tag_t *tag, int shape_id,
         return 1;
     }
     result = swf_tag_shape_apply_matrix_factor(tag->detail, shape_id,
-                                                     scale_x, scale_y,
-                                                     rotate_rad,
-                                                     trans_x, trans_y);
+                                               bitmap_id,
+                                               scale_x, scale_y,
+                                               rotate_rad,
+                                               trans_x, trans_y);
     if (result == 0) {
         free(tag->data);
         tag->data = NULL;
@@ -1001,7 +1018,7 @@ swf_tag_apply_shape_matrix_factor(swf_tag_t *tag, int shape_id,
 }
 
 int
-swf_tag_apply_shape_rect_factor(swf_tag_t *tag, int shape_id,
+swf_tag_apply_shape_rect_factor(swf_tag_t *tag, int shape_id, int bitmap_id,
                                   double scale_x, double scale_y,
                                   signed int trans_x, signed int trans_y,
                                   struct swf_object_ *swf) {
@@ -1022,7 +1039,7 @@ swf_tag_apply_shape_rect_factor(swf_tag_t *tag, int shape_id,
         fprintf(stderr, "swf_tag_apply_shape_rect_factor: swf_tag_create_input_detail failed\n");
         return 1;
     }
-    result = swf_tag_shape_apply_rect_factor(tag->detail, shape_id,
+    result = swf_tag_shape_apply_rect_factor(tag->detail, shape_id, bitmap_id,
                                              scale_x, scale_y,
                                              trans_x, trans_y);
     if (result == 0) {
@@ -1034,7 +1051,7 @@ swf_tag_apply_shape_rect_factor(swf_tag_t *tag, int shape_id,
 }
 
 int
-swf_tag_apply_shape_type_tilled(swf_tag_t *tag, int shape_id,
+swf_tag_apply_shape_type_tilled(swf_tag_t *tag, int shape_id, int bitmap_id,
                                 struct swf_object_ *swf) {
     int result;
     if (tag == NULL) {
@@ -1053,7 +1070,7 @@ swf_tag_apply_shape_type_tilled(swf_tag_t *tag, int shape_id,
         fprintf(stderr, "swf_tag_apply_shape_rect_factor: swf_tag_create_input_detail failed\n");
         return 1;
     }
-    result = swf_tag_shape_apply_type_tilled(tag->detail, shape_id);
+    result = swf_tag_shape_apply_type_tilled(tag->detail, shape_id, bitmap_id);
     if (result == 0) {
         free(tag->data);
         tag->data = NULL;
